@@ -303,9 +303,11 @@ std::map<std::string, DescHandlerFunc> GetHandlers() {
           {variables::kRebase, DefaultHandler},
           {variables::kWalkKeys, DefaultHandler},
           {variables::kWeakFrameworks, DefaultHandler},
+          {variables::kWeakLibraries, DefaultHandler},
           {variables::kWriteOutputConversion, DefaultHandler},
           {variables::kRustCrateName, DefaultHandler},
           {variables::kRustCrateRoot, DefaultHandler},
+          {variables::kRustflags, DefaultHandler},
           {variables::kSwiftModuleName, DefaultHandler},
           {variables::kSwiftBridgeHeader, DefaultHandler},
           {variables::kMnemonic, DefaultHandler},
@@ -399,8 +401,10 @@ bool PrintTarget(const Target* target,
   HandleProperty(variables::kLibDirs, handler_map, v, dict);
   HandleProperty(variables::kDataKeys, handler_map, v, dict);
   HandleProperty(variables::kRebase, handler_map, v, dict);
+  HandleProperty(variables::kRustflags, handler_map, v, dict);
   HandleProperty(variables::kWalkKeys, handler_map, v, dict);
   HandleProperty(variables::kWeakFrameworks, handler_map, v, dict);
+  HandleProperty(variables::kWeakLibraries, handler_map, v, dict);
   HandleProperty(variables::kWriteOutputConversion, handler_map, v, dict);
 
 #undef HandleProperty
@@ -463,7 +467,9 @@ bool PrintConfig(const Config* config,
   HandleProperty(variables::kLibDirs, handler_map, v, dict);
   HandleProperty(variables::kPrecompiledHeader, handler_map, v, dict);
   HandleProperty(variables::kPrecompiledSource, handler_map, v, dict);
+  HandleProperty(variables::kRustflags, handler_map, v, dict);
   HandleProperty(variables::kWeakFrameworks, handler_map, v, dict);
+  HandleProperty(variables::kWeakLibraries, handler_map, v, dict);
 
 #undef HandleProperty
 
@@ -526,6 +532,7 @@ Possibilities for <what to show>
   visibility
   walk_keys
   weak_frameworks
+  weak_libraries
 
   runtime_deps
       Compute all runtime deps for the given target. This is a computed list
@@ -620,6 +627,25 @@ Examples
       each one was set from.
 )";
 
+class PrintCallbackHolder {
+ public:
+  PrintCallbackHolder() {}
+  ~PrintCallbackHolder() {
+    if (_settings.has_value()) {
+      _settings.value()->swap_print_callback(_callback.value());
+    }
+  }
+  void SwapCallbacks(BuildSettings* settings,
+                     BuildSettings::PrintCallback new_callback) {
+    _settings = settings;
+    _callback = settings->swap_print_callback(new_callback);
+  }
+
+ private:
+  std::optional<BuildSettings*> _settings;
+  std::optional<BuildSettings::PrintCallback> _callback;
+};
+
 int RunDesc(const std::vector<std::string>& args) {
   if (args.size() != 2 && args.size() != 3) {
     Err(Location(), "Unknown command format. See \"gn help desc\"",
@@ -631,6 +657,16 @@ int RunDesc(const std::vector<std::string>& args) {
 
   // Deliberately leaked to avoid expensive process teardown.
   Setup* setup = new Setup;
+
+  bool json = cmdline->GetSwitchValueString("format") == "json";
+  PrintCallbackHolder print_callback_holder;
+  if (json) {
+    // Silence all output while running desc if outputting to json.
+    BuildSettings* settings = &setup->build_settings();
+    print_callback_holder.SwapCallbacks(settings,
+                                        [](const std::string& str) {});
+  }
+
   if (!setup->DoSetup(args[0], false))
     return 1;
   if (!setup->Run())
@@ -654,8 +690,6 @@ int RunDesc(const std::vector<std::string>& args) {
   if (args.size() == 3)
     what_to_print = args[2];
 
-  bool json = cmdline->GetSwitchValueString("format") == "json";
-
   if (target_matches.empty() && config_matches.empty()) {
     OutputString(
         "The input " + args[1] + " matches no targets, configs or files.\n",
