@@ -5,6 +5,7 @@
 #include "gn/header_checker.h"
 
 #include <algorithm>
+#include <unordered_map>
 
 #include "base/containers/queue.h"
 #include "base/files/file_util.h"
@@ -151,6 +152,7 @@ bool HeaderChecker::Run(const std::vector<const Target*>& to_check,
 
 void HeaderChecker::RunCheckOverFiles(const FileMap& files, bool force_check) {
   WorkerPool pool;
+  task_count_.Increment();
 
   for (const auto& file : files) {
     // Only check C-like source files (RC files also have includes).
@@ -181,6 +183,8 @@ void HeaderChecker::RunCheckOverFiles(const FileMap& files, bool force_check) {
     }
   }
 
+  task_count_.Decrement();
+
   // Wait for all tasks posted by this method to complete.
   std::unique_lock<std::mutex> auto_lock(lock_);
   while (!task_count_.IsZero())
@@ -558,7 +562,7 @@ bool HeaderChecker::IsDependencyOf(const Target* search_for,
   // a shortest dependency chain (in reverse order) from search_from to
   // search_for.
 
-  std::map<const Target*, ChainLink> breadcrumbs;
+  std::unordered_map<const Target*, ChainLink> breadcrumbs;
   base::queue<ChainLink> work_queue;
   work_queue.push(ChainLink(search_from, true));
 
@@ -582,7 +586,7 @@ bool HeaderChecker::IsDependencyOf(const Target* search_for,
 
     // Always consider public dependencies as possibilities.
     for (const auto& dep : target->public_deps()) {
-      if (breadcrumbs.insert(std::make_pair(dep.ptr, cur_link)).second)
+      if (breadcrumbs.emplace(dep.ptr, cur_link).second)
         work_queue.push(ChainLink(dep.ptr, true));
     }
 
@@ -593,7 +597,7 @@ bool HeaderChecker::IsDependencyOf(const Target* search_for,
       // public/private-ness.
       first_time = false;
       for (const auto& dep : target->private_deps()) {
-        if (breadcrumbs.insert(std::make_pair(dep.ptr, cur_link)).second)
+        if (breadcrumbs.emplace(dep.ptr, cur_link).second)
           work_queue.push(ChainLink(dep.ptr, false));
       }
     }
@@ -654,8 +658,9 @@ Err HeaderChecker::MakeUnreachableError(const InputFile& source_file,
       1)
     msg += "at least one of ";
   msg += "which should somehow be reachable.\n";
-  msg += "This might be a false alarm if you are using conditional include. "
-         "See \"gn help nogncheck\".";
+  msg +=
+      "This might be a false alarm if you are using conditional include. "
+      "See \"gn help nogncheck\".";
 
   // Danger: must call CreatePersistentRange to put in Err.
   return Err(CreatePersistentRange(source_file, range), "Include not allowed.",
