@@ -7,6 +7,7 @@
 
 #include <set>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "base/gtest_prod_util.h"
@@ -410,6 +411,15 @@ class Target : public Item {
     return runtime_outputs_;
   }
 
+  // The module name for the target.
+  std::string module_name() const {
+    return module_name_override_.empty() ? label().name()
+                                         : module_name_override_;
+  }
+  void set_module_name(std::string module_name) {
+    module_name_override_ = std::move(module_name);
+  }
+
   // Computes and returns the outputs of this target expressed as SourceFiles.
   //
   // For binary target this depends on the tool for this target so the toolchain
@@ -488,7 +498,6 @@ class Target : public Item {
   bool CheckTestonly(Err* err) const;
   bool CheckAssertNoDeps(Err* err) const;
   void CheckSourcesGenerated() const;
-  void CheckSourceGenerated(const SourceFile& source) const;
   bool CheckSourceSetLanguages(Err* err) const;
 
   OutputType output_type_ = UNKNOWN;
@@ -498,6 +507,8 @@ class Target : public Item {
   std::string output_extension_;
   bool output_extension_set_ = false;
 
+  std::string module_name_override_;
+
   FileList sources_;
   SourceFileTypeSet source_types_used_;
   bool all_headers_public_ = true;
